@@ -1,3 +1,5 @@
+/* $Xorg: sm_client.c,v 1.4 2001/02/09 02:03:30 xorgcvs Exp $ */
+
 /*
 
 Copyright 1993, 1998  The Open Group
@@ -33,27 +35,27 @@ in this Software without prior written authorization from The Open Group.
 #endif
 #include <X11/SM/SMlib.h>
 #include "SMlibint.h"
+#include "globals.h"
 
-int 	_SmcOpcode = 0;
-int 	_SmsOpcode = 0;
-
-SmsNewClientProc _SmsNewClientProc;
-SmPointer        _SmsNewClientData;
-
-SmcErrorHandler _SmcErrorHandler = _SmcDefaultErrorHandler;
-SmsErrorHandler _SmsErrorHandler = _SmsDefaultErrorHandler;
-
-
-static void
-set_callbacks(SmcConn smcConn, unsigned long mask, SmcCallbacks *callbacks);
+static void set_callbacks();
 
 
 SmcConn
-SmcOpenConnection(char *networkIdsList, SmPointer context,
-		  int xsmpMajorRev, int xsmpMinorRev,
-		  unsigned long mask, SmcCallbacks *callbacks,
-		  const char *previousId, char **clientIdRet,
-		  int errorLength, char *errorStringRet)
+SmcOpenConnection (networkIdsList, context,
+    xsmpMajorRev, xsmpMinorRev, mask, callbacks,
+    previousId, clientIdRet, errorLength, errorStringRet)
+
+char 		*networkIdsList;
+SmPointer	context;
+int		xsmpMajorRev;
+int		xsmpMinorRev;
+unsigned long   mask;
+SmcCallbacks	*callbacks;
+char 		*previousId;
+char 		**clientIdRet;
+int  		errorLength;
+char 		*errorStringRet;
+
 {
     SmcConn			smcConn;
     IceConn			iceConn;
@@ -65,21 +67,11 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
     char			*release = NULL;
     smRegisterClientMsg 	*pMsg;
     char 			*pData;
-    unsigned int		extra, len;
+    int				extra, len;
     IceReplyWaitInfo		replyWait;
     _SmcRegisterClientReply	reply;
     Bool			gotReply, ioErrorOccured;
 
-    const char *auth_names[] = {"MIT-MAGIC-COOKIE-1"};
-    IcePoAuthProc auth_procs[] = {_IcePoMagicCookie1Proc};
-    int auth_count = 1;
-
-    IcePoVersionRec versions[] = {
-        {SmProtoMajor, SmProtoMinor, _SmcProcessMessage}
-    };
-    int version_count = 1;
-
-
     *clientIdRet = NULL;
 
     if (errorStringRet && errorLength > 0)
@@ -90,20 +82,16 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 	/*
 	 * For now, there is only one version of XSMP, so we don't
 	 * have to check {xsmpMajorRev, xsmpMinorRev}.  In the future,
-	 * we will check against versions and generate the list
+	 * we will check against _SmcVersions and generate the list
 	 * of versions the application actually supports.
 	 */
 
 	if ((_SmcOpcode = IceRegisterForProtocolSetup ("XSMP",
-	    SmVendorString, SmReleaseString, version_count, versions,
-            auth_count, auth_names, auth_procs, NULL)) < 0)
+	    SmVendorString, SmReleaseString, _SmVersionCount, _SmcVersions,
+            _SmAuthCount, _SmAuthNames, _SmcAuthProcs, NULL)) < 0)
 	{
-	    if (errorStringRet && errorLength > 0) {
-		strncpy (errorStringRet,
-			 "Could not register XSMP protocol with ICE",
-			 errorLength);
-		errorStringRet[errorLength - 1] = '\0';
-	    }
+	    strncpy (errorStringRet,
+	        "Could not register XSMP protocol with ICE", errorLength);
 
 	    return (NULL);
 	}
@@ -113,12 +101,10 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
     {
 	if ((ids = (char *) getenv ("SESSION_MANAGER")) == NULL)
 	{
-	    if (errorStringRet && errorLength > 0) {
-		strncpy (errorStringRet,
-			 "SESSION_MANAGER environment variable not defined",
-			 errorLength);
-		errorStringRet[errorLength - 1] = '\0';
-	    }
+	    strncpy (errorStringRet,
+	        "SESSION_MANAGER environment variable not defined",
+		errorLength);
+
 	    return (NULL);
 	}
     }
@@ -133,12 +119,9 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 	return (NULL);
     }
 
-    if ((smcConn = malloc (sizeof (struct _SmcConn))) == NULL)
+    if ((smcConn = (SmcConn) malloc (sizeof (struct _SmcConn))) == NULL)
     {
-	if (errorStringRet && errorLength > 0) {
-	    strncpy (errorStringRet, "Can't malloc", errorLength);
-	    errorStringRet[errorLength - 1] = '\0';
-	}
+	strncpy (errorStringRet, "Can't malloc", errorLength);
 	IceCloseConnection (iceConn);
 	return (NULL);
     }
@@ -153,7 +136,7 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 	setupstat == IceProtocolSetupIOError)
     {
 	IceCloseConnection (iceConn);
-	free (smcConn);
+	free ((char *) smcConn);
 	return (NULL);
     }
     else if (setupstat == IceProtocolAlreadyActive)
@@ -164,12 +147,9 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 	 * may not already have XSMP active on it.
 	 */
 
-	free (smcConn);
-	if (errorStringRet && errorLength > 0) {
-	    strncpy (errorStringRet, "Internal error in IceOpenConnection",
-		     errorLength);
-	    errorStringRet[errorLength - 1] = '\0';
-	}
+	free ((char *) smcConn);
+	strncpy (errorStringRet, "Internal error in IceOpenConnection",
+		errorLength);
 	return (NULL);
     }
 
@@ -195,22 +175,16 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
      * Now register the client
      */
 
-    if (!previousId)
-	previousId = "";
-    len = strlen (previousId);
+    len = previousId ? strlen (previousId) : 0;
     extra = ARRAY8_BYTES (len);
 
     IceGetHeaderExtra (iceConn, _SmcOpcode, SM_RegisterClient,
 	SIZEOF (smRegisterClientMsg), WORD64COUNT (extra),
 	smRegisterClientMsg, pMsg, pData);
 
-    if (pData != NULL) {
-	STORE_ARRAY8 (pData, len, previousId);
-	IceFlush (iceConn);
-    }
-    else {
-	SEND_ARRAY8 (iceConn, len, previousId);
-    }
+    STORE_ARRAY8 (pData, len, previousId);
+
+    IceFlush (iceConn);
 
     replyWait.sequence_of_request = IceLastSentSequenceNumber (iceConn);
     replyWait.major_opcode_of_request = _SmcOpcode;
@@ -227,14 +201,12 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 
 	if (ioErrorOccured)
 	{
-	    if (errorStringRet && errorLength > 0) {
-		strncpy (errorStringRet, "IO error occurred opening connection",
-			 errorLength);
-		errorStringRet[errorLength - 1] = '\0';
-	    }
+	    strncpy (errorStringRet, "IO error occured opening connection",
+		errorLength);
+
 	    free (smcConn->vendor);
 	    free (smcConn->release);
-	    free (smcConn);
+	    free ((char *) smcConn);
 
 	    return (NULL);
 	}
@@ -248,7 +220,10 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 
 		*clientIdRet = reply.client_id;
 
-		smcConn->client_id = strdup (*clientIdRet);
+		smcConn->client_id = (char *) malloc (
+		    strlen (*clientIdRet) + 1);
+
+		strcpy (smcConn->client_id, *clientIdRet);
 	    }
 	    else
 	    {
@@ -264,13 +239,9 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 		    SIZEOF (smRegisterClientMsg), WORD64COUNT (extra),
 		    smRegisterClientMsg, pMsg, pData);
 
-		if (pData != NULL) {
-		    STORE_ARRAY8 (pData, 0, "");
-		    IceFlush (iceConn);
-		}
-		else {
-		    SEND_ARRAY8 (iceConn, 0, "");
-		}
+		STORE_ARRAY8 (pData, 0, NULL);
+
+		IceFlush (iceConn);
 
 		replyWait.sequence_of_request =
 		    IceLastSentSequenceNumber (iceConn);
@@ -286,7 +257,12 @@ SmcOpenConnection(char *networkIdsList, SmPointer context,
 
 
 SmcCloseStatus
-SmcCloseConnection(SmcConn smcConn, int count, char **reasonMsgs)
+SmcCloseConnection (smcConn, count, reasonMsgs)
+
+SmcConn smcConn;
+int	count;
+char    **reasonMsgs;
+
 {
     IceConn			iceConn = smcConn->iceConn;
     smCloseConnectionMsg 	*pMsg;
@@ -304,24 +280,13 @@ SmcCloseConnection(SmcConn smcConn, int count, char **reasonMsgs)
 	SIZEOF (smCloseConnectionMsg), WORD64COUNT (extra),
 	smCloseConnectionMsg, pMsg, pData);
 
-    if (pData != NULL) {
-	STORE_CARD32 (pData, (CARD32) count);
-	STORE_CARD32 (pData, (CARD32) 0); /* padding */
+    STORE_CARD32 (pData, count);
+    pData += 4;
 
-	for (i = 0; i < count; i++)
-	    STORE_ARRAY8 (pData, strlen (reasonMsgs[i]), reasonMsgs[i]);
+    for (i = 0; i < count; i++)
+	STORE_ARRAY8 (pData, strlen (reasonMsgs[i]), reasonMsgs[i]); 
 
-	IceFlush (iceConn);
-    } else {
-	CARD32 count_header[2] = {
-	    (CARD32) count,
-	    (CARD32) 0 /* padding */
-	};
-	IceWriteData32 (iceConn, 8, count_header);
-
-	for (i = 0; i < count; i++)
-	    SEND_ARRAY8 (iceConn, strlen (reasonMsgs[i]), reasonMsgs[i]);
-    }
+    IceFlush (iceConn);
 
     IceProtocolShutdown (iceConn, _SmcOpcode);
     IceSetShutdownNegotiation (iceConn, False);
@@ -336,22 +301,6 @@ SmcCloseConnection(SmcConn smcConn, int count, char **reasonMsgs)
     if (smcConn->client_id)
 	free (smcConn->client_id);
 
-    if (smcConn->interact_waits)
-    {
-	_SmcInteractWait *ptr = smcConn->interact_waits;
-	_SmcInteractWait *next;
-
-	while (ptr)
-	{
-	    next = ptr->next;
-	    free (ptr);
-	    ptr = next;
-	}
-    }
-
-    if (smcConn->phase2_wait)
-	free (smcConn->phase2_wait);
-
     if (smcConn->prop_reply_waits)
     {
 	_SmcPropReplyWait *ptr = smcConn->prop_reply_waits;
@@ -360,13 +309,13 @@ SmcCloseConnection(SmcConn smcConn, int count, char **reasonMsgs)
 	while (ptr)
 	{
 	    next = ptr->next;
-	    free (ptr);
+	    free ((char *) ptr);
 	    ptr = next;
 	}
-
+	
     }
 
-    free (smcConn);
+    free ((char *) smcConn);
 
     if (closeStatus == IceClosedNow)
 	statusRet = SmcClosedNow;
@@ -381,7 +330,12 @@ SmcCloseConnection(SmcConn smcConn, int count, char **reasonMsgs)
 
 
 void
-SmcModifyCallbacks(SmcConn smcConn, unsigned long mask, SmcCallbacks *callbacks)
+SmcModifyCallbacks (smcConn, mask, callbacks)
+
+SmcConn    	smcConn;
+unsigned long 	mask;
+SmcCallbacks	*callbacks;
+
 {
     set_callbacks (smcConn, mask, callbacks);
 }
@@ -389,13 +343,18 @@ SmcModifyCallbacks(SmcConn smcConn, unsigned long mask, SmcCallbacks *callbacks)
 
 
 void
-SmcSetProperties(SmcConn smcConn, int numProps, SmProp **props)
+SmcSetProperties (smcConn, numProps, props)
+
+SmcConn    	smcConn;
+int      	numProps;
+SmProp       	**props;
+
 {
     IceConn		iceConn = smcConn->iceConn;
     smSetPropertiesMsg	*pMsg;
     char		*pBuf;
     char		*pStart;
-    unsigned int	bytes;
+    int			bytes;
 
     IceGetHeader (iceConn, _SmcOpcode, SM_SetProperties,
 	SIZEOF (smSetPropertiesMsg), smSetPropertiesMsg, pMsg);
@@ -404,7 +363,6 @@ SmcSetProperties(SmcConn smcConn, int numProps, SmProp **props)
     pMsg->length += WORD64COUNT (bytes);
 
     pBuf = pStart = IceAllocScratch (iceConn, bytes);
-    memset(pStart, 0, bytes);
 
     STORE_LISTOF_PROPERTY (pBuf, numProps, props);
 
@@ -415,7 +373,12 @@ SmcSetProperties(SmcConn smcConn, int numProps, SmProp **props)
 
 
 void
-SmcDeleteProperties(SmcConn smcConn, int numProps, char **propNames)
+SmcDeleteProperties (smcConn, numProps, propNames)
+
+SmcConn smcConn;
+int     numProps;
+char	**propNames;
+
 {
     IceConn			iceConn = smcConn->iceConn;
     smDeletePropertiesMsg 	*pMsg;
@@ -431,37 +394,30 @@ SmcDeleteProperties(SmcConn smcConn, int numProps, char **propNames)
 	SIZEOF (smDeletePropertiesMsg), WORD64COUNT (extra),
 	smDeletePropertiesMsg, pMsg, pData);
 
-    if (pData != NULL) {
-	STORE_CARD32 (pData, (CARD32) numProps);
-	STORE_CARD32 (pData, (CARD32) 0); /* padding */
+    STORE_CARD32 (pData, numProps);
+    pData += 4;
 
-	for (i = 0; i < numProps; i++)
-	    STORE_ARRAY8 (pData, strlen (propNames[i]), propNames[i]);
+    for (i = 0; i < numProps; i++)
+	STORE_ARRAY8 (pData, strlen (propNames[i]), propNames[i]); 
 
-	IceFlush (iceConn);
-    }
-    else {
-	CARD32 count_header[2] = {
-	    (CARD32) numProps,
-	    (CARD32) 0 /* padding */
-	};
-	IceWriteData32 (iceConn, 8, count_header);
-
-	for (i = 0; i < numProps; i++)
-	    SEND_ARRAY8 (iceConn, strlen (propNames[i]), propNames[i]);
-    }
+    IceFlush (iceConn);
 }
 
 
 
 Status
-SmcGetProperties(SmcConn smcConn, SmcPropReplyProc propReplyProc,
-		 SmPointer clientData)
+SmcGetProperties (smcConn, propReplyProc, clientData)
+
+SmcConn		 smcConn;
+SmcPropReplyProc propReplyProc;
+SmPointer	 clientData;
+
 {
     IceConn		iceConn = smcConn->iceConn;
     _SmcPropReplyWait 	*wait, *ptr;
 
-    if ((wait = malloc (sizeof (_SmcPropReplyWait))) == NULL)
+    if ((wait = (_SmcPropReplyWait *) malloc (
+	sizeof (_SmcPropReplyWait))) == NULL)
     {
 	return (0);
     }
@@ -488,14 +444,20 @@ SmcGetProperties(SmcConn smcConn, SmcPropReplyProc propReplyProc,
 
 
 Status
-SmcInteractRequest(SmcConn smcConn, int dialogType,
-		   SmcInteractProc interactProc, SmPointer clientData)
+SmcInteractRequest (smcConn, dialogType, interactProc, clientData)
+
+SmcConn 	smcConn;
+int		dialogType;
+SmcInteractProc	interactProc;
+SmPointer	clientData;
+
 {
     IceConn			iceConn = smcConn->iceConn;
     smInteractRequestMsg	*pMsg;
     _SmcInteractWait 		*wait, *ptr;
 
-    if ((wait = malloc (sizeof (_SmcInteractWait))) == NULL)
+    if ((wait = (_SmcInteractWait *) malloc (
+	sizeof (_SmcInteractWait))) == NULL)
     {
 	return (0);
     }
@@ -526,7 +488,11 @@ SmcInteractRequest(SmcConn smcConn, int dialogType,
 
 
 void
-SmcInteractDone(SmcConn smcConn, Bool cancelShutdown)
+SmcInteractDone (smcConn, cancelShutdown)
+
+SmcConn smcConn;
+Bool 	cancelShutdown;
+
 {
     IceConn		iceConn = smcConn->iceConn;
     smInteractDoneMsg	*pMsg;
@@ -542,8 +508,16 @@ SmcInteractDone(SmcConn smcConn, Bool cancelShutdown)
 
 
 void
-SmcRequestSaveYourself(SmcConn smcConn, int saveType, Bool shutdown,
-		       int interactStyle, Bool fast, Bool global)
+SmcRequestSaveYourself (smcConn, saveType, shutdown, interactStyle,
+	fast, global)
+
+SmcConn smcConn;
+int	saveType;
+Bool 	shutdown;
+int	interactStyle;
+Bool	fast;
+Bool	global;
+
 {
     IceConn			iceConn = smcConn->iceConn;
     smSaveYourselfRequestMsg	*pMsg;
@@ -563,9 +537,12 @@ SmcRequestSaveYourself(SmcConn smcConn, int saveType, Bool shutdown,
 
 
 Status
-SmcRequestSaveYourselfPhase2(SmcConn smcConn,
-			     SmcSaveYourselfPhase2Proc saveYourselfPhase2Proc,
-			     SmPointer clientData)
+SmcRequestSaveYourselfPhase2 (smcConn, saveYourselfPhase2Proc, clientData)
+
+SmcConn 			smcConn;
+SmcSaveYourselfPhase2Proc	saveYourselfPhase2Proc;
+SmPointer			clientData;
+
 {
     IceConn		iceConn = smcConn->iceConn;
     _SmcPhase2Wait 	*wait;
@@ -574,7 +551,8 @@ SmcRequestSaveYourselfPhase2(SmcConn smcConn,
 	wait = smcConn->phase2_wait;
     else
     {
-	if ((wait = malloc (sizeof (_SmcPhase2Wait))) == NULL)
+	if ((wait = (_SmcPhase2Wait *) malloc (
+	    sizeof (_SmcPhase2Wait))) == NULL)
 	{
 	    return (0);
 	}
@@ -594,7 +572,11 @@ SmcRequestSaveYourselfPhase2(SmcConn smcConn,
 
 
 void
-SmcSaveYourselfDone(SmcConn smcConn, Bool success)
+SmcSaveYourselfDone (smcConn, success)
+
+SmcConn smcConn;
+Bool	success;
+
 {
     IceConn			iceConn = smcConn->iceConn;
     smSaveYourselfDoneMsg	*pMsg;
@@ -610,7 +592,12 @@ SmcSaveYourselfDone(SmcConn smcConn, Bool success)
 
 
 static void
-set_callbacks(SmcConn smcConn, unsigned long mask, SmcCallbacks *callbacks)
+set_callbacks (smcConn, mask, callbacks)
+
+SmcConn    	smcConn;
+unsigned long 	mask;
+SmcCallbacks	*callbacks;
+
 {
     if (mask & SmcSaveYourselfProcMask)
     {
